# -*- coding: utf-8 -*-
# vim: set ts=4
#
# Copyright 2021-present Linaro Limited
#
# SPDX-License-Identifier: MIT

import os
import re
import sys
import time
from urllib.parse import urlparse

from tuxrun.requests import requests_get
from tuxrun.utils import ProgressIndicator, NoProgressIndicator
from tuxrun.xdg import get_cache_dir


KERNELS = {
    "qemu-armv5": "https://storage.tuxboot.com/armv5/zImage",
    "qemu-armv7": "https://storage.tuxboot.com/armv7/zImage",
    "qemu-armv7be": "https://storage.tuxboot.com/armv7be/zImage",
    "qemu-arm64": "https://storage.tuxboot.com/arm64/Image",
    "qemu-arm64be": "https://storage.tuxboot.com/arm64be/Image",
    "qemu-i386": "https://storage.tuxboot.com/i386/bzImage",
    "qemu-mips32": "https://storage.tuxboot.com/mips32/vmlinux",
    "qemu-mips32el": "https://storage.tuxboot.com/mips32el/vmlinux",
    "qemu-mips64": "https://storage.tuxboot.com/mips64/vmlinux",
    "qemu-mips64el": "https://storage.tuxboot.com/mips64el/vmlinux",
    "qemu-ppc32": "https://storage.tuxboot.com/ppc32/uImage",
    "qemu-ppc64": "https://storage.tuxboot.com/ppc64/vmlinux",
    "qemu-ppc64le": "https://storage.tuxboot.com/ppc64le/vmlinux",
    "qemu-riscv64": "https://storage.tuxboot.com/riscv64/Image",
    "qemu-s390": "https://storage.tuxboot.com/s390/bzImage",
    "qemu-sh4": "https://storage.tuxboot.com/sh4/zImage",
    "qemu-sparc64": "https://storage.tuxboot.com/sparc64/vmlinux",
    "qemu-x86_64": "https://storage.tuxboot.com/x86_64/bzImage",
}


ROOTFS = {
    "qemu-armv5": "https://storage.tuxboot.com/armv5/rootfs.ext4.zst",
    "qemu-armv7": "https://storage.tuxboot.com/armv7/rootfs.ext4.zst",
    "qemu-armv7be": "https://storage.tuxboot.com/armv7be/rootfs.ext4.zst",
    "qemu-arm64": "https://storage.tuxboot.com/arm64/rootfs.ext4.zst",
    "qemu-arm64be": "https://storage.tuxboot.com/arm64be/rootfs.ext4.zst",
    "qemu-i386": "https://storage.tuxboot.com/i386/rootfs.ext4.zst",
    "qemu-mips32el": "https://storage.tuxboot.com/mips32el/rootfs.ext4.zst",
    "qemu-mips32": "https://storage.tuxboot.com/mips32/rootfs.ext4.zst",
    "qemu-mips64el": "https://storage.tuxboot.com/mips64el/rootfs.ext4.zst",
    "qemu-mips64": "https://storage.tuxboot.com/mips64/rootfs.ext4.zst",
    "qemu-ppc32": "https://storage.tuxboot.com/ppc32/rootfs.ext4.zst",
    "qemu-ppc64": "https://storage.tuxboot.com/ppc64/rootfs.ext4.zst",
    "qemu-ppc64le": "https://storage.tuxboot.com/ppc64le/rootfs.ext4.zst",
    "qemu-riscv64": "https://storage.tuxboot.com/riscv64/rootfs.ext4.zst",
    "qemu-s390": "https://storage.tuxboot.com/s390/rootfs.ext4.zst",
    "qemu-sh4": "https://storage.tuxboot.com/sh4/rootfs.ext4.zst",
    "qemu-sparc64": "https://storage.tuxboot.com/sparc64/rootfs.ext4.zst",
    "qemu-x86_64": "https://storage.tuxboot.com/x86_64/rootfs.ext4.zst",
}


TEST_DEFINITIONS = "https://storage.tuxboot.com/test-definitions/2022.01.tar.zst"


def get_rootfs(
    device, rootfs: str = None, progress: ProgressIndicator = NoProgressIndicator()
) -> str:
    return __download_and_cache__(rootfs or ROOTFS[device], progress)


def get_test_definitions(progress: ProgressIndicator = NoProgressIndicator()):
    return __download_and_cache__(TEST_DEFINITIONS, progress)


def __download_and_cache__(
    url: str, progress: ProgressIndicator = NoProgressIndicator()
):
    parsed = urlparse(url)
    if parsed.scheme not in ["http", "https"]:
        return url

    key = re.sub(r"[:/]", "_", url)

    cache_dir = get_cache_dir().resolve() / "assets"
    cache_dir.mkdir(parents=True, exist_ok=True)
    cache = cache_dir / key
    cache_etag_file = cache_dir / (key + ".etag")

    expired = False
    if cache.exists():
        timestamp = os.stat(cache).st_mtime
        now = time.time()
        timeout = 6 * 60 * 60  # 6 hours

        expired = (now - timestamp) > timeout
        if not expired:
            return str(cache)

    try:
        response = requests_get(url, allow_redirects=True, stream=True)
        response.raise_for_status()
        etag = str(response.headers["ETag"])
    except KeyError:
        print("Missing ETag, continuing without cache", file=sys.stderr)
        return url
    except Exception as e:
        if cache.exists():
            print(e, "Continuing with cached version of the file", file=sys.stderr)
            return str(cache)
        raise

    if cache_etag_file.exists():
        cache_etag = cache_etag_file.read_text()
    else:
        cache_etag = None
    if cache_etag == etag:
        response.close()
        return str(cache)
    else:
        cache_etag_file.write_text(etag)

    size = int(response.headers.get("Content-Length", "0"))
    with cache.open("wb") as data:
        n = 0
        for chunk in response.iter_content(chunk_size=4096):
            if chunk:
                n += data.write(chunk)
                if size:
                    progress.progress(100 * n / size)

    if size:
        progress.finish()

    return str(cache.resolve())



# -*- coding: utf-8 -*-
# vim: set ts=4
#
# Copyright 2021-present Linaro Limited
#
# SPDX-License-Identifier: MIT

from functools import lru_cache
from pathlib import Path

import jinja2


BASE = (Path(__file__) / "..").resolve()


@lru_cache(maxsize=None)
def jobs():
    return jinja2.Environment(
        autoescape=False,
        trim_blocks=True,
        loader=jinja2.FileSystemLoader(str(BASE / "jobs")),
        undefined=jinja2.StrictUndefined,
    )


@lru_cache(maxsize=None)
def devices():
    return jinja2.Environment(
        autoescape=False,
        trim_blocks=True,
        loader=jinja2.FileSystemLoader(str(BASE / "devices")),
    )


@lru_cache(maxsize=None)
def dispatchers():
    return jinja2.Environment(
        autoescape=False,
        trim_blocks=True,
        loader=jinja2.FileSystemLoader(str(BASE / "dispatchers")),
    )


@lru_cache(maxsize=None)
def tests():
    return jinja2.Environment(
        autoescape=False,
        trim_blocks=True,
        loader=jinja2.FileSystemLoader(str(BASE / "tests")),
    )


@lru_cache(maxsize=None)
def wrappers():
    return jinja2.Environment(
        autoescape=False,
        trim_blocks=True,
        loader=jinja2.FileSystemLoader(str(BASE / "wrappers")),
    )



# -*- coding: utf-8 -*-
# vim: set ts=4
#
# Copyright 2021-present Linaro Limited
#
# SPDX-License-Identifier: MIT

from typing import Dict, List, Optional

from tuxrun import templates
from tuxrun.devices import Device
from tuxrun.exceptions import InvalidArgument


class FVPDevice(Device):
    mandatory = [
        "ap_romfw",
        "mcp_fw",
        "mcp_romfw",
        "rootfs",
        "scp_fw",
        "scp_romfw",
        "fip",
    ]

    prompts: List[str] = []
    auto_login: Dict[str, str] = {}
    boot_timeout = 20
    support_tests = False
    rootfs: Optional[str] = None

    def validate(
        self,
        ap_romfw,
        mcp_fw,
        mcp_romfw,
        rootfs,
        scp_fw,
        scp_romfw,
        parameters,
        tests,
        fip,
        **kwargs,
    ):
        invalid_args = ["--" + k.replace("_", "-") for k in kwargs if kwargs[k]]
        if len(invalid_args) > 0:
            raise InvalidArgument(
                f"Invalid option(s) for fvp devices: {', '.join(sorted(invalid_args))}"
            )

        args = locals()
        missing_args = [
            "--" + k.replace("_", "-") for k in self.mandatory if not args[k]
        ]
        if len(missing_args) > 0:
            raise InvalidArgument(
                f"Missing option(s) for fvp devices: {', '.join(sorted(missing_args))}"
            )

        if tests and not self.support_tests:
            raise InvalidArgument("Tests are not supported on this device")

        if self.rootfs and rootfs:
            raise InvalidArgument("Invalid option for this fvp device: --rootfs")

        for test in tests:
            test.validate(device=self, parameters=parameters, **kwargs)

    def definition(self, **kwargs):
        kwargs = kwargs.copy()

        # Options that can *not* be updated
        kwargs["prompts"] = self.prompts.copy()
        kwargs["auto_login"] = self.auto_login.copy()
        kwargs["support_tests"] = self.support_tests

        kwargs["rootfs"] = self.rootfs if self.rootfs else kwargs.get("rootfs")
        kwargs["boot_timeout"] = self.boot_timeout

        # render the template
        tests = [
            t.render(
                tmpdir=kwargs["tmpdir"],
                parameters=kwargs["parameters"],
            )
            for t in kwargs["tests"]
        ]
        return (
            templates.jobs().get_template("fvp.yaml.jinja2").render(**kwargs)
            + "\n"
            + "".join(tests)
        )

    def device_dict(self, context):
        return templates.devices().get_template("fvp.yaml.jinja2").render(**context)


class FVPMorelloAndroid(FVPDevice):
    name = "fvp-morello-android"

    prompts = ["console:/ "]
    support_tests = True


class FVPMorelloBusybox(FVPDevice):
    name = "fvp-morello-busybox"

    prompts = ["/ # "]
    support_tests = False


class FVPMorelloOE(FVPDevice):
    name = "fvp-morello-oe"

    prompts = ["root@morello-fvp:~# "]
    support_tests = True


class FVPMorelloUbuntu(FVPDevice):
    name = "fvp-morello-ubuntu"

    mandatory = ["ap_romfw", "mcp_fw", "mcp_romfw", "scp_fw", "scp_romfw", "fip"]

    prompts = ["morello@morello-server:"]
    auto_login = {
        "login_prompt": "morello-server login:",
        "username": "morello",
        "password_prompt": "Password:",
        "password": "morello",
    }
    boot_timeout = 60
    rootfs = "https://storage.tuxboot.com/fvp-morello-ubuntu/ubuntu.satadisk.xz"



# -*- coding: utf-8 -*-
# vim: set ts=4
#
# Copyright 2022-present Linaro Limited
#
# SPDX-License-Identifier: MIT

from tuxrun.exceptions import InvalidArgument
from tuxrun.tests import Test


class KSelfTest(Test):
    device = "qemu-*"
    cmdfile: str = ""
    need_test_definition = True

    def validate(self, device, parameters, **kwargs):
        super().validate(device=device, parameters=parameters, **kwargs)

        # CPUPOWER is only for x86_64 and i386
        required = ["KSELFTEST"]
        if device in ["qemu-i386", "qemu-x86_64"]:
            required = ["CPUPOWER", "KSELFTEST"]

        missing = set(required) - set(parameters.keys())
        if missing:
            raise InvalidArgument(f"Missing --parameters {', '.join(sorted(missing))}")

    def render(self, **kwargs):
        kwargs["name"] = self.name
        kwargs["timeout"] = self.timeout
        kwargs["cmdfile"] = (
            self.cmdfile if self.cmdfile else self.name.replace("ltp-", "")
        )

        if kwargs["device"].name in ["qemu-i386", "qemu-x86_64"]:
            kwargs["overlays"].append(
                ("cpupower", kwargs["parameters"]["CPUPOWER"], "/")
            )
        kwargs["overlays"].append(
            ("kselftest", kwargs["parameters"]["KSELFTEST"], "/opt/kselftest_intree/")
        )

        return self._render("kselftest.yaml.jinja2", **kwargs)


class KSelftestGpio(KSelfTest):
    name = "kselftest-gpio"
    cmdfile = "gpio"
    timeout = 5


class KSelftestIPC(KSelfTest):
    name = "kselftest-ipc"
    cmdfile = "ipc"
    timeout = 5


class KSelftestIR(KSelfTest):
    name = "kselftest-ir"
    cmdfile = "ir"
    timeout = 5


class KSelftestKcmp(KSelfTest):
    name = "kselftest-kcmp"
    cmdfile = "kcmp"
    timeout = 5


class KSelftestKexec(KSelfTest):
    name = "kselftest-kexec"
    cmdfile = "kexec"
    timeout = 5


class KSelftestRseq(KSelfTest):
    name = "kselftest-rseq"
    cmdfile = "rseq"
    timeout = 5


class KSelftestRtc(KSelfTest):
    name = "kselftest-rtc"
    cmdfile = "rtc"
    timeout = 5



from hashlib import sha1
from pathlib import Path
import pytest
import subprocess
import time
from tuxrun.assets import get_rootfs


seven_hours = 7 * 60 * 60


def rewind_timestamp(filename, interval):
    past = time.time() - interval
    subprocess.check_call(["touch", "-d", f"@{past}", str(filename)])


class TestGetRootfs:
    def test_local_file(self):
        assert get_rootfs("qemu-arm64", "/path/to/file") == "/path/to/file"

    def test_default(self):
        rootfs = get_rootfs("qemu-arm64")
        assert Path(rootfs).exists()

    def test_downloads_remote_file(self, get, mocker):
        rootfs = get_rootfs("qemu-arm64", "https://example.com/rootfs.img")
        assert Path(rootfs).exists()
        get.assert_called()
        assert get.call_args[0][0] == "https://example.com/rootfs.img"

    def test_caches_downloads(self, get):
        get_rootfs("qemu-arm64")
        get_rootfs("qemu-arm64")
        assert get.call_count == 1

    def test_writes_data_to_disk(self, response):
        response.iter_content.return_value = [b"123"]
        rootfs = get_rootfs("qemu-x86_64")
        assert Path(rootfs).read_text() == "123"

    @pytest.fixture
    def build_response(self, mocker):
        def __build_response__(data):
            r = mocker.MagicMock()
            r.iter_content.return_value = [data]
            data_hash = sha1(data).hexdigest()
            r.headers = {"ETag": f'"{data_hash}"', "Content-Length": str(len(data))}
            return r

        return __build_response__

    def test_update_after_6h_if_etag_changed(self, get, build_response):
        response1 = build_response(b"123")
        response2 = build_response(b"456")
        get.side_effect = [response1, response2]

        rootfs = get_rootfs("qemu-x86_64")
        rewind_timestamp(rootfs, seven_hours)

        rootfs = get_rootfs("qemu-x86_64")
        assert Path(rootfs).read_text() == "456"

    def test_uses_cache_after_6h_with_same_etag(self, response):
        rootfs = get_rootfs("qemu-x86_64")
        rewind_timestamp(rootfs, seven_hours)
        response.iter_content.assert_called()

        response.reset_mock()
        get_rootfs("qemu-x86_64")
        response.iter_content.assert_not_called()

    def test_uses_cached_version_when_download_fails(self, get, response):
        get.side_effect = [response, RuntimeError("BOOM")]
        r1 = get_rootfs("qemu-x86_64")
        rewind_timestamp(r1, seven_hours)
        r2 = get_rootfs("qemu-x86_64")
        assert r1 == r2

    def test_fails_without_cache_when_download_fails(self, get, response):
        get.side_effect = RuntimeError("BOOM")
        with pytest.raises(RuntimeError):
            get_rootfs("qemu-x86_64")


if __name__ == "__main__":
    rootfs = get_rootfs("qemu-x86_64")
    print(rootfs)



#!/usr/bin/python3
import argparse
import fnmatch
from pathlib import Path
import shutil
import subprocess
import sys
import tempfile

from typing import Dict, Any
from tuxrun.devices import Device
from tuxrun.tests import Test
from tuxrun.yaml import yaml_load


###########
# Helpers #
###########
def get_results(tmpdir: Path) -> Dict:
    required_keys = set(["msg", "lvl", "dt"])
    res: Dict[Any, Any] = {}
    data = yaml_load((tmpdir / "logs.yaml").read_text(encoding="utf-8"))

    if data is None:
        return {}

    for line in data:
        if not isinstance(line, dict):
            continue
        if not required_keys.issubset(set(line.keys())):
            continue
        if line["lvl"] != "results":
            continue
        definition = line["msg"]["definition"]
        case = line["msg"]["case"]
        del line["msg"]["definition"]
        del line["msg"]["case"]
        res.setdefault(definition, {})[case] = line["msg"]

    return res


def get_simple_results(res: Dict) -> Dict:
    results = {
        "boot": res.get("lava", {}).get("login-action", {}).get("result", "fail")
    }
    for name in res:
        if name == "lava":
            continue
        key = "_".join(name.split("_")[1:])
        if all(res[name][case]["result"] in ["skip", "pass"] for case in res[name]):
            results[key] = "pass"
        else:
            results[key] = "fail"
    return results


def get_job_result(results: Dict, simple_results: Dict) -> str:
    # lava.job is missing: error
    lava = results.get("lava", {}).get("job")
    if lava is None:
        return "error"

    if lava["result"] == "fail":
        if lava.get("error_type") == "Job":
            return "fail"
        return "error"

    if all(v == "pass" for (k, v) in simple_results.items()):
        return "pass"
    return "fail"


def run(device, test, runtime, debug):
    tmpdir = Path(tempfile.mkdtemp(prefix="tuxrun-"))

    args = [
        "python3",
        "-m",
        "tuxrun",
        "--device",
        device,
        "--runtime",
        runtime,
        "--log-file",
        str(tmpdir / "logs.yaml"),
    ]
    if test:
        args.extend(["--tests", test])

    if device == "fvp-morello-android":
        args.extend(
            [
                "--mcp-fw",
                "https://storage.tuxboot.com/fvp-morello-android/mcp_fw.bin",
                "--mcp-romfw",
                "https://storage.tuxboot.com/fvp-morello-android/mcp_romfw.bin",
                "--rootfs",
                "https://storage.tuxboot.com/fvp-morello-android/android-nano.img.xz",
                "--scp-fw",
                "https://storage.tuxboot.com/fvp-morello-android/scp_fw.bin",
                "--scp-romfw",
                "https://storage.tuxboot.com/fvp-morello-android/scp_romfw.bin",
                "--fip",
                "https://storage.tuxboot.com/fvp-morello-busybox/fip.bin",
                "--image",
                "tuxrun:fvp",
            ]
        )
        if test == "binder":
            args.extend(
                [
                    "--parameters",
                    "USERDATA=https://storage.tuxboot.com/fvp-morello-android/userdata.tar.xz",
                ]
            )
        elif test == "bionic":
            args.extend(
                [
                    "--parameters",
                    "USERDATA=https://storage.tuxboot.com/fvp-morello-android/userdata.tar.xz",
                ]
            )
        elif test == "boringssl":
            args.extend(
                [
                    "--parameters",
                    "SYSTEM_URL=https://storage.tuxboot.com/fvp-morello-android/system.tar.xz",
                ]
            )
        elif test == "compartment":
            args.extend(
                [
                    "--parameters",
                    "USERDATA=https://storage.tuxboot.com/fvp-morello-android/userdata.tar.xz",
                ]
            )
        elif test == "libjpeg-turbo":
            args.extend(
                [
                    "--parameters",
                    "LIBJPEG_TURBO_URL=https://storage.tuxboot.com/fvp-morello-android/libjpeg-turbo.tar.xz",
                    "--parameters",
                    "SYSTEM_URL=https://storage.tuxboot.com/fvp-morello-android/system.tar.xz",
                ]
            )
        elif test == "libpng":
            args.extend(
                [
                    "--parameters",
                    "PNG_URL=https://storage.tuxboot.com/fvp-morello-android/png-testfiles.tar.xz",
                    "--parameters",
                    "SYSTEM_URL=https://storage.tuxboot.com/fvp-morello-android/system.tar.xz",
                ]
            )
        elif test == "libpdfium":
            args.extend(
                [
                    "--parameters",
                    "PDFIUM_URL=https://storage.tuxboot.com/fvp-morello-android/pdfium-testfiles.tar.xz",
                    "--parameters",
                    "SYSTEM_URL=https://storage.tuxboot.com/fvp-morello-android/system.tar.xz",
                ]
            )
        elif test == "lldb":
            args.extend(
                [
                    "--parameters",
                    "LLDB_URL=https://storage.tuxboot.com/fvp-morello-android/lldb_tests.tar.xz",
                    "--parameters",
                    "TC_URL=https://storage.tuxboot.com/fvp-morello-android/morello-clang.tar.xz",
                ]
            )
        elif test == "logd":
            args.extend(
                [
                    "--parameters",
                    "USERDATA=https://storage.tuxboot.com/fvp-morello-android/userdata.tar.xz",
                ]
            )
        elif test == "zlib":
            args.extend(
                [
                    "--parameters",
                    "SYSTEM_URL=https://storage.tuxboot.com/fvp-morello-android/system.tar.xz",
                ]
            )
    elif device == "fvp-morello-busybox":
        args.extend(
            [
                "--mcp-fw",
                "https://storage.tuxboot.com/fvp-morello-busybox/mcp_fw.bin",
                "--mcp-romfw",
                "https://storage.tuxboot.com/fvp-morello-busybox/mcp_romfw.bin",
                "--rootfs",
                "https://storage.tuxboot.com/fvp-morello-busybox/busybox.img.xz",
                "--scp-fw",
                "https://storage.tuxboot.com/fvp-morello-busybox/scp_fw.bin",
                "--scp-romfw",
                "https://storage.tuxboot.com/fvp-morello-busybox/scp_romfw.bin",
                "--fip",
                "https://storage.tuxboot.com/fvp-morello-busybox/fip.bin",
                "--image",
                "tuxrun:fvp",
            ]
        )
    elif device == "fvp-morello-ubuntu":
        args.extend(
            [
                "--mcp-fw",
                "https://storage.tuxboot.com/fvp-morello-ubuntu/mcp_fw.bin",
                "--mcp-romfw",
                "https://storage.tuxboot.com/fvp-morello-ubuntu/mcp_romfw.bin",
                "--scp-fw",
                "https://storage.tuxboot.com/fvp-morello-ubuntu/scp_fw.bin",
                "--scp-romfw",
                "https://storage.tuxboot.com/fvp-morello-ubuntu/scp_romfw.bin",
                "--fip",
                "https://storage.tuxboot.com/fvp-morello-ubuntu/fip.bin",
                "--image",
                "tuxrun:fvp",
            ]
        )

    try:
        ret = subprocess.call(args)
        if ret != 0:
            print(f"Command return non-zero exist status {ret}")
            print((tmpdir / "logs.yaml").read_text(encoding="utf-8"))
            return ret

        results = get_results(tmpdir)
        simple_results = get_simple_results(results)
        result = get_job_result(results, simple_results)

        if debug:
            print("Results:")
            for res in results:
                print(f"* {res}: {results[res]}")

            print("\nSimple results:")
            for res in simple_results:
                print(f"* {res}: {simple_results[res]}")
            print(f"Result {result}")
        else:
            print(f"{result}")
        assert result == "pass"
    finally:
        shutil.rmtree(tmpdir)


##############
# Entrypoint #
##############
def main():
    parser = argparse.ArgumentParser(description="Integration tests")
    parser.add_argument("--devices", nargs="+", required=True, help="device")
    parser.add_argument(
        "--tests",
        default=["boot"],
        nargs="+",
        help="tests",
    )
    parser.add_argument("--debug", default=False, action="store_true", help="debug")
    parser.add_argument(
        "--runtime",
        default="podman",
        choices=["docker", "null", "podman"],
        help="Runtime",
    )
    options = parser.parse_args()

    if len(options.devices) == 1 and "*" in options.devices[0]:
        pat = options.devices[0]
        options.devices = [d for d in Device.list() if fnmatch.fnmatch(d, pat)]

    for device in options.devices:
        tests = options.tests.copy()
        if tests == ["all"]:
            tests = ["boot"] + Test.list(device=device)
        for test in tests:
            print(f"=> {device} x {test}")
            if run(
                device, "" if test == "boot" else test, options.runtime, options.debug
            ):
                return 1
            print("")


if __name__ == "__main__":
    sys.exit(main())



# -*- coding: utf-8 -*-
# vim: set ts=4
#
# Copyright 2021-present Linaro Limited
#
# SPDX-License-Identifier: MIT

from tuxrun.tests import Test


class Command(Test):
    device = "qemu-*"
    name = "command"
    timeout = 2

    def render(self, **kwargs):
        kwargs["name"] = self.name
        kwargs["timeout"] = self.timeout

        return self._render("command.yaml.jinja2", **kwargs)



from tuxrun.results import Results
from tuxrun.tests import Test


def test_returns_0_by_default():
    results = Results([])
    results.__data__ = {"lava": {}}
    assert results.ret() == 0


def gen_test(name, result, suite_name="mytestsuite"):
    return f'{{ "lvl": "results", "msg": {{"definition": "{suite_name}", "case": "{name}", "result": "{result}"}}}}'


def test_returns_0_with_no_failures():
    t1 = Test(timeout=None)
    t1.name = "mytestsuite"
    results = Results([t1])
    results.parse(gen_test("test1", "pass"))
    results.parse(gen_test("test2", "pass"))
    results.parse(gen_test("job", "pass", suite_name="lava"))
    assert results.ret() == 0


def test_returns_1_on_failure():
    results = Results([])
    results.parse(gen_test("test1", "pass", suite_name="lava"))
    results.parse(gen_test("test2", "fail", suite_name="lava"))
    assert results.ret() == 1


def test_returns_2_on_missing_test():
    t1 = Test(timeout=None)
    t1.name = "mytestsuite"
    results = Results([t1])
    results.parse(gen_test("test1", "pass", suite_name="lava"))
    assert results.ret() == 2


def test_returns_invalid_logs():
    results = Results([])
    results.parse("{")
    results.parse('{ "lvl": "results", "msg": {"case": "tux", "result": "pass"}}')


def test_data():
    results = Results([])
    results.parse(gen_test("test1", "pass"))
    assert results.data["mytestsuite"]["test1"]["result"] == "pass"



# -*- coding: utf-8 -*-
# vim: set ts=4
#
# Copyright 2021-present Linaro Limited
#
# SPDX-License-Identifier: MIT

import contextlib
import json
from pathlib import Path

from tuxrun.requests import requests_get


class InvalidTuxBuild(Exception):
    pass


class TuxBuild:
    Invalid = InvalidTuxBuild

    @classmethod
    def parse(cls, url, data):
        try:
            metadata = json.loads(data)
        except json.JSONDecodeError as e:
            raise cls.Invalid(f"Invalid metadata.json: {e}")

        try:
            target_arch = metadata["build"]["target_arch"]
        except KeyError:
            raise cls.Invalid("{url}/metadata.json is invalid")

        kernel = modules = None
        with contextlib.suppress(KeyError):
            kernel = url + "/" + metadata["results"]["artifacts"]["kernel"][0]
        with contextlib.suppress(IndexError, KeyError):
            modules = url + "/" + metadata["results"]["artifacts"]["modules"][0]

        return (target_arch, kernel, modules)


class TuxBuildBuild(TuxBuild):
    def __init__(self, url):
        ret = requests_get(f"{url}/metadata.json")
        if ret.status_code != 200:
            raise self.Invalid(f"{url}/metadata.json is missing")

        (self.target_arch, self.kernel, self.modules) = TuxBuild.parse(url, ret.text)


class TuxMakeBuild(TuxBuild):
    def __init__(self, directory):
        self.location = Path(directory).resolve()
        metadata_file = self.location / "metadata.json"
        if not self.location.is_dir():
            raise self.Invalid(f"{directory} is not a directory")
        if not metadata_file.exists():
            raise self.Invalid(
                f"{directory} is not a valid TuxMake artifacts directory: missing metadata.json"
            )

        (self.target_arch, self.kernel, self.modules) = TuxBuild.parse(
            f"file://{self.location}", metadata_file.read_text(encoding="utf-8")
        )



# -*- coding: utf-8 -*-
# vim: set ts=4
#
# Copyright 2021-present Linaro Limited
#
# SPDX-License-Identifier: MIT


class TuxRunException(Exception):
    def __str__(self):
        name = super().__str__()
        if hasattr(self, "msg"):
            return self.msg.format(name=name)
        else:
            return name


class InvalidArgument(TuxRunException):
    pass



from tuxrun.writer import Writer


def test_write_log_file(tmp_path):
    log_file = tmp_path / "logs.yaml"
    with Writer(log_file) as writer:
        writer.write(
            '{"lvl": "info", "msg": "Hello, world", "dt": "2021-04-08T18:42:25.139513"}'
        )

    log_file.read_text(
        encoding="utf-8"
    ) == '{"lvl": "info", "msg": "Hello, world", "dt": "2021-04-08T18:42:25.139513"}\n'


def test_write_stdout(capsys):
    with Writer(None) as writer:
        writer.write(
            '{"lvl": "info", "msg": "Hello, world", "dt": "2021-04-08T18:42:25.139513"}'
        )

    out, err = capsys.readouterr()
    assert "\x1b[0m \x1b[1;37mHello, world\x1b[0m\n" in out


def test_write_stdout_feedback(capsys):
    with Writer(None) as writer:
        writer.write(
            '{"lvl": "feedback", "msg": "Hello, world", "dt": "2021-04-08T18:42:25.139513", "ns": "testing"}'
        )

    out, err = capsys.readouterr()
    assert "\x1b[0m <\x1b[0;33mtesting\x1b[0m> \x1b[0;33mHello, world\x1b[0m\n" in out


def test_writer_invalid_yaml(capsys, tmpdir):
    data = '{"lvl": "feedback", "msg": "Hello, world", "dt": "2021-04-08T18:42:25.139513", "ns": "testing"}'
    with Writer(tmpdir / "logs.yaml") as writer:
        writer.write("{")
        writer.write("hello world")
        writer.write("{}")
        writer.write(data)
        writer.write("{hello: world}")
    out, err = capsys.readouterr()
    assert (
        out
        == """{
hello world
{}
{hello: world}
"""
    )
    assert err == ""
    assert (tmpdir / "logs.yaml").read_text(encoding="utf-8") == f"- {data}\n"



# -*- coding: utf-8 -*-
# vim: set ts=4
#
# Copyright 2021-present Linaro Limited
#
# SPDX-License-Identifier: MIT

import logging
import re

import yaml

from tuxrun.yaml import yaml_load


LOG = logging.getLogger("tuxrun")
PATTERN = re.compile(r"^(\d+_)")


class Results:
    def __init__(self, tests):
        self.__data__ = {}
        self.__post_processed = False
        self.__tests__ = set(["lava"] + [t.name for t in tests])
        self.__ret__ = 0

    def parse(self, line):
        try:
            data = yaml_load(line)
        except yaml.YAMLError:
            LOG.debug(line)
            return
        if not data or not isinstance(data, dict):
            LOG.debug(line)
            return
        if data.get("lvl") != "results":
            return

        test = data.get("msg")
        if not set(["case", "definition"]).issubset(test.keys()):
            LOG.debug(line)
            return

        definition = re.sub(PATTERN, "", test.pop("definition"))
        case = re.sub(PATTERN, "", test.pop("case"))
        self.__data__.setdefault(definition, {})[case] = test
        if test["result"] == "fail":
            self.__ret__ = 1

    def __post_process(self):
        if self.__post_processed:
            return
        self.__post_processed = True

        if self.__tests__ != set(self.__data__.keys()):
            self.__ret__ = 2

    @property
    def data(self):
        self.__post_process()
        return self.__data__

    def ret(self):
        self.__post_process()
        return self.__ret__



# -*- coding: utf-8 -*-
# vim: set ts=4
#
# Copyright 2021-present Linaro Limited
#
# SPDX-License-Identifier: MIT

from tuxrun.exceptions import InvalidArgument


def subclasses(cls):
    return set(cls.__subclasses__()).union(
        [s for c in cls.__subclasses__() for s in subclasses(c)]
    )


class Device:
    name = ""

    @classmethod
    def select(cls, name):
        for subclass in subclasses(cls):

            if subclass.name == name:
                return subclass
        raise InvalidArgument(
            f"Unknown device {name}. Available: {', '.join(cls.list())}"
        )

    @classmethod
    def list(cls):
        return sorted([s.name for s in subclasses(cls) if s.name])

    def validate(self, **kwargs):
        raise NotImplementedError()  # pragma: no cover

    def definition(self, **kwargs):
        raise NotImplementedError()  # pragma: no cover

    def device_dict(self, context):
        raise NotImplementedError()  # pragma: no cover


import tuxrun.devices.fvp  # noqa: E402
import tuxrun.devices.qemu  # noqa: E402,F401
